@@ -1,22 +1,11 @@
-from __future__ import print_function
-from __future__ import absolute_import
-
-try:
-    from urllib.request import urlopen
-except:
-    from urllib2 import urlopen
-
-from future import standard_library
-standard_library.install_aliases()
-from builtins import str
 import argparse
 import os
 import subprocess
-import traceback
-from . import utils
+import urllib2
+import utils
 import sys
 from shutil import copyfile
-from .agent import run_agent
+from agent import run_agent
 from time import sleep
 
 
@@ -24,7 +13,9 @@ def main():
     s3_base_link = 'https://s3.amazonaws.com/spotinst-public/services/spotinst-agent'
     s3_base_workers_link = s3_base_link + "/workers"
     s3_base_collectors_link = s3_base_link + "/collectors"
+    s3_base_initiator_link = s3_base_link + "/initiators"
     modules_path = '/etc/spotinst/agent/modules'
+    initiator_modules_path = '/etc/spotinst/agent/initiators/modules'
     instance_details_path = '/etc/spotinst'
 
     results = parse_arguments()
@@ -55,6 +46,12 @@ def main():
         else:
             print ("Please enter the name of the collector which you want to add.")
 
+    elif action == "add-initiator":
+        if argument is not None:
+            install_initiator(argument, initiator_modules_path, s3_base_initiator_link)
+        else:
+            print("Please enter the name of the initiator which you want to add.")
+
     elif action == "run":
         run_agent(debug, verbose, cloud_provider)
 
@@ -75,7 +72,7 @@ def main():
             try_num += 1
             print ("Initializing agent")
             install_succeed = handle_cloud_provider(cloud_provider, instance_details_path) and install_and_init_agent(
-                argument, second_argument, third_argument, modules_path)
+                argument, second_argument, third_argument, modules_path, initiator_modules_path)
 
         if not install_succeed:
             print("init failed")
@@ -83,10 +80,11 @@ def main():
         else:
             print("Spotinst agent installed successfully")
     else:
-        print ("Unrecognized action. Please use one of the following : add-worker, add-collector, configure")
+        print("Unrecognized action. Please use one of the following : add-worker, add-collector, add-initiator "
+              "configure")
 
 
-def install_and_init_agent(token, account_id, custom_host, modules_path):
+def install_and_init_agent(token, account_id, custom_host, modules_path, initiator_modules_path):
     # Paths
     runscript = None
     try:
@@ -94,10 +92,9 @@ def install_and_init_agent(token, account_id, custom_host, modules_path):
         script_full_path = os.path.join(dirname, 'data/installation/agent-init.sh')
         spotinst_agent_path = os.path.join(dirname, 'agent.py')
         create_agent_local_folders()
-        handle_base_modules(dirname, modules_path)
+        handle_base_modules(dirname, modules_path, initiator_modules_path)
         handle_yml_file(dirname)
-        handle_credentials(account_id, script_full_path, spotinst_agent_path, token)
-        handle_custom_host(custom_host)
+        handle_credentials(account_id, script_full_path, spotinst_agent_path, token, custom_host)
         handle_agent_path(spotinst_agent_path)
         runscript = subprocess.Popen(script_full_path)
         runscript.wait()
@@ -121,7 +118,7 @@ def handle_cloud_provider(cloud_provider, instance_details_path):
         os.chmod(full_path, 0o644)
         return True
     except IOError:
-        print("Unable to create /etc/spotinst/cloud_provider.txt")
+        print ("Unable to create /etc/spotinst/cloud_provider.txt")
         if cloud_provider_file is not None:
             cloud_provider_file.close()
         return False
@@ -138,11 +135,16 @@ def create_agent_local_folders():
         os.makedirs('/var/log/spotinst')
 
 
-def handle_base_modules(dirname, modules_path):
+def handle_base_modules(dirname, modules_path, initiator_modules_path):
     basemodule_yml_path = os.path.join(dirname, 'data/configuration/basemodule.yml')
     utils_path = os.path.join(dirname, 'utils.py')
     basemodule_template_path = os.path.join(dirname, 'basemodule.py')
 
+    if not os.path.exists(initiator_modules_path):
+        os.makedirs(initiator_modules_path)
+    copyfile(basemodule_template_path, initiator_modules_path + '/basemodule.py')
+    copyfile(basemodule_yml_path, initiator_modules_path + '/basemodule.yml')
+    copyfile(utils_path, initiator_modules_path + '/utils.py')
     if not os.path.exists(modules_path):
         os.makedirs(modules_path)
     copyfile(basemodule_template_path, modules_path + '/basemodule.py')
@@ -157,12 +159,15 @@ def handle_yml_file(dirname):
         copyfile(default_agent_yml, default_yml_destination)
 
 
-def handle_credentials(account_id, script_full_path, spotinst_agent_path, token):
+def handle_credentials(account_id, script_full_path, spotinst_agent_path, token, host):
     if token is not None:
         print ("Creating Spotinst credentials directory")
 
+        if host is None:
+            #if the customer didn't supply custom host set the default
+            host = 'api.spotinst.io'
         creds_file, creds_path = create_creds_file()
-        write_creds_path(account_id, creds_file, token)
+        write_creds_path(account_id, creds_file, token, host)
         os.chmod(creds_path, 0o777)
 
     else:
@@ -170,20 +175,19 @@ def handle_credentials(account_id, script_full_path, spotinst_agent_path, token)
         creds = utils.retrieve_creds()
         token = creds["token"]
         account_id = creds["account_id"]
+        host = creds["host"]
     os.putenv("SPOTINST_TOKEN", token)
     os.putenv("SPOTINST_ACCOUNT_ID", account_id)
+    os.putenv("CUSTOM_HOST", host)
     os.putenv("SPOTINST_AGENT_PATH", spotinst_agent_path)
     os.chmod(script_full_path, 0o777)
 
-def handle_custom_host(custom_host):
-    if custom_host is not None:
-        os.putenv("CUSTOM_HOST", custom_host)
 
-def write_creds_path(account_id, creds_file, token):
+def write_creds_path(account_id, creds_file, token, host):
     if account_id is not None:
-        creds_file.write('token = ' + token + '\n' + 'account_id = ' + account_id)
+        creds_file.write('token = ' + token + '\n' + 'account_id = ' + account_id + '\n' + 'host = ' + host)
     else:
-        creds_file.write('token = ' + token)
+        creds_file.write('token = ' + token + '\n' + 'host = ' + host)
 
 
 def create_creds_file():
@@ -248,12 +252,21 @@ def install_collector(collector_name, modules_path, s3_base_collectors_link):
         get_collector_files(collector_name, modules_path, s3_base_collectors_link)
 
     except Exception as e:
-        print ("Failed to install Collector: %s. %s" % (e, traceback.format_exc()))
+        print ("Could not get collector " + collector_name + " Exception : " + str(e))
+
+
+def install_initiator(initiator_name, initiator_modules_path, s3_base_initiator_link):
+    print("adding initiator " + initiator_name)
+    try:
+        get_initiator_files(initiator_name, initiator_modules_path, s3_base_initiator_link)
+
+    except Exception as e:
+        print("Could not get initiator " + initiator_name + " Exception : " + str(e))
 
 
 def get_worker_files(modules_path, s3_base_workers_link, worker_name):
-    py_response = urlopen(s3_base_workers_link + '/' + worker_name + '/' + worker_name + '.py')
-    yml_response = urlopen(s3_base_workers_link + '/' + worker_name + '/' + worker_name + '.yml')
+    py_response = urllib2.urlopen(s3_base_workers_link + '/' + worker_name + '/' + worker_name + '.py')
+    yml_response = urllib2.urlopen(s3_base_workers_link + '/' + worker_name + '/' + worker_name + '.yml')
 
     with open(modules_path + '/' + worker_name + '.py', 'wb') as worker:
         worker.write(py_response.read())
@@ -263,18 +276,26 @@ def get_worker_files(modules_path, s3_base_workers_link, worker_name):
 
 def get_collector_files(collector_name, modules_path, s3_base_collectors_link):
     with open(modules_path + '/' + collector_name + '.py', 'wb') as collector:
-        print(s3_base_collectors_link + '/' + collector_name + '/' + collector_name + '.py')
-        response = urlopen(s3_base_collectors_link + '/' + collector_name + '/' + collector_name + '.py')
+        response = urllib2.urlopen(s3_base_collectors_link + '/' + collector_name + '/' + collector_name + '.py')
         collector.write(response.read())
     with open(modules_path + '/' + collector_name + '.yml', 'wb') as collector_cfg:
-        response = urlopen(s3_base_collectors_link + '/' + collector_name + '/' + collector_name + '.yml')
+        response = urllib2.urlopen(s3_base_collectors_link + '/' + collector_name + '/' + collector_name + '.yml')
         collector_cfg.write(response.read())
 
 
+def get_initiator_files(initiator_name, initiator_modules_path, s3_base_initiator_link):
+    with open(initiator_modules_path + '/' + initiator_name + '.py', 'wb') as initiator:
+        response = urllib2.urlopen(s3_base_initiator_link + '/' + initiator_name + '/' + initiator_name + '.py')
+        initiator.write(response.read())
+    with open(initiator_modules_path + '/' + initiator_name + '.yml', 'wb') as initiator_cfg:
+        response = urllib2.urlopen(s3_base_initiator_link + '/' + initiator_name + '/' + initiator_name + '.yml')
+        initiator_cfg.write(response.read())
+
+
 def handle_prerequisites(s3_base_module_link, module_name):
     try:
-        response = urlopen(s3_base_module_link + '/' + module_name + '/' + module_name + '.req')
-        requirements_file = response.read().decode('utf-8')
+        response = urllib2.urlopen(s3_base_module_link + '/' + module_name + '/' + module_name + '.req')
+        requirements_file = response.read()
         requirements_names = requirements_file.split("\n")
         print ("This module requires the following packages to be installed:")
         for requirement in requirements_names:
@@ -297,6 +318,5 @@ def handle_prerequisites(s3_base_module_link, module_name):
                 print ("Some prerequisites for this module have not been met. " \
                        "Please install the following before proceeding: " + str(missing_packages))
                 sys.exit(1)
-    except Exception as e:
-        print ("Failed to handle prerequisites: %s. %s" % (e, traceback.format_exc()))
+    except Exception:
         pass
