@@ -1,6 +1,3 @@
-from __future__ import absolute_import
-from builtins import str
-from builtins import object
 import inspect
 import logging
 import os
@@ -8,9 +5,9 @@ import sys
 import time
 import traceback
 
-from .scheduler import ThreadedScheduler, method
-from .utils import load_class_from_name
-from . import basemodule
+from scheduler import ThreadedScheduler, method
+from utils import load_class_from_name
+import basemodule
 
 
 class TaskManager(object):
@@ -28,6 +25,17 @@ class TaskManager(object):
         self.instance_details = instance_details
 
     def start(self):
+        initiators_folder = '/etc/spotinst/agent/initiators/modules'
+        self.log.debug("service is %s" % self.enable)
+
+        # Search for initiators in folder
+        initiators = self.load_modules(initiators_folder)
+
+        # Setup Modules
+        for initiator in initiators.values():
+            c = self.init_module(initiator['cls'])
+            self.schedule_initiator_module(c, initiator)
+
         modules_folder = '/etc/spotinst/agent/modules'
         self.log.debug("service is %s" % self.enable)
 
@@ -35,7 +43,7 @@ class TaskManager(object):
         modules = self.load_modules(modules_folder)
 
         # Setup Modules
-        for module in list(modules.values()):
+        for module in modules.values():
             # Initialize Module
             c = self.init_module(module['cls'])
             # Schedule Module
@@ -76,7 +84,7 @@ class TaskManager(object):
                 modules = self.load_modules(modules_folder)
 
                 # Setup Modules
-                for module in list(modules.values()):
+                for module in modules.values():
                     # Initialize Module
                     c = self.init_module(module['cls'])
                     # Schedule Module
@@ -124,11 +132,11 @@ class TaskManager(object):
                 mod_modification_time_str = str(mod_modification_time)
                 self.log.info("Py file found: %s" % modname)
 
-                modules_list = [x.lower() for x in list(self.tasks.keys())]
+                modules_list = [x.lower() for x in self.tasks.keys()]
                 if modname in modules_list:
                     if self.tasks[modname]['modification_time'] == mod_modification_time_str:
                         self.log.info("Module '{0}' already loaded".format(modname))
-                        continue
+                #         continue
 
                 if modname == "utils" or modname == "basemodule":
                     continue
@@ -137,8 +145,7 @@ class TaskManager(object):
                 now = time.time()
                 if now - mod_modification_time < 1:
                     self.log.info("Not loading file since it was created too recently: modified at %s, now is %s" % (mod_modification_time, now))
-                    continue
-
+                #     continue
                 try:
                     # Import the module
                     self.log.info("Importing %s" % modname)
@@ -154,8 +161,9 @@ class TaskManager(object):
                     attr = getattr(mod, attrname)
                     # Only attempt to load classes that are infact classes but not the base Module class
                     self.log.debug("Checking " + str(modname) + " Attribute " + str(attrname))
+                    self.log.info("Checking " + str(modname) + " Attribute " + str(attrname))
 
-                    if inspect.isclass(attr) and isModule(attr) and attr != basemodule.Module and attrname != "Module":
+                    if inspect.isclass(attr) and isModule(attr) and attr != basemodule.Module:
                         # Get class name
                         fqcn = '.'.join([modname, attrname])
                         self.log.info("Module loaded : " + fqcn)
@@ -205,7 +213,7 @@ class TaskManager(object):
             return
 
         # Get Module schedule
-        for name, schedule in list(c.get_schedule().items()):
+        for name, schedule in c.get_schedule().items():
             # Get scheduler args
             func, args, splay, interval = schedule
 
@@ -227,6 +235,41 @@ class TaskManager(object):
             # Add task to list
             self.tasks[module['file_name']] = {'task': task, 'modification_time': module['modification_time']}
 
+    def schedule_initiator_module(self, c, module):
+        """
+        Schedule module
+        """
+        if c is None:
+            self.log.warning("Skipped loading invalid Module: %s" % c.__class__.__name__)
+            return
+
+        if not c.get_enable():
+            self.log.info("Skipped loading Module '%s' since it is disabled" % c.__class__.__name__)
+            return
+
+        # Get Module schedule
+        for name, schedule in c.get_schedule().items():
+            # Get scheduler args
+            func, args, splay, interval = schedule
+
+            # Check if a module with same name has already been scheduled
+            if module['file_name'] in self.tasks:
+                try:
+                    self.scheduler.cancel(self.tasks[module['file_name']]['task'])
+                    # Log
+                    self.log.info("Canceled task: %s" % name)
+                except ValueError as e:
+                    self.log.error("Unable to cancel task '{0}' - {1}".format(self.tasks[name], e))
+
+            # Schedule Collector
+            self.log.info("Scheduled task {0} to run every {1}s (delayed of {2}s)".format(name, interval, splay))
+            task = self.scheduler.add_single_task(func, name, splay, method.sequential,
+                                                  args, None)
+
+            self.log.debug("Scheduled task: %s" % name)
+            # Add task to list
+            self.tasks[module['file_name']] = {'task': task, 'modification_time': module['modification_time']}
+
     def stop(self):
         """
         Close and stop all tasks.
@@ -236,7 +279,7 @@ class TaskManager(object):
 
         self.log.info('Stopped task scheduler.')
 
-        for key, task in self.tasks.items():
+        for key, task in self.tasks.iteritems():
             try:
                 self.scheduler.cancel(task['task'])
                 # Log
