@@ -1,22 +1,12 @@
-from __future__ import print_function
-
-try:
-    from urllib.request import Request, urlopen
-    from urllib.error import HTTPError, URLError
-except:
-    from urllib2 import Request, urlopen, HTTPError, URLError
-
-from future import standard_library
-standard_library.install_aliases()
-from builtins import str
 import inspect
-import http.client
+import httplib
 import json
 import logging
 import os
 import sys
 from _ssl import SSLError
 from json import loads
+from urllib2 import Request, urlopen, HTTPError, URLError
 import yaml
 
 sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
@@ -42,13 +32,13 @@ def safe_get_json(json_string, key, log=logging.getLogger('spotinst-agent')):
     return ret
 
 
-def safe_read_simple_url(url, log=logging.getLogger('spotinst-agent')):
+def safe_read_simple_url(url, log=logging.getLogger('spotinst-agent'), headers={}):
     try:
         log.debug("reading simple {} ...".format(url))
-        req = Request(url)
+        req = Request(url, headers=headers)
         req.add_header('Content-Type', 'application/json')
         response = urlopen(req, timeout=10)
-        return response.read().decode('utf-8')
+        return response.read()
     except HTTPError as e:
         log.error("The server can't fulfill the GET {} request. [reason: {}]".format(url, e.reason))
     except URLError as e:
@@ -61,6 +51,33 @@ def safe_read_simple_url(url, log=logging.getLogger('spotinst-agent')):
         except NameError:
             pass
 
+
+def safe_put_simple_url(url, log=logging.getLogger('spotinst-agent'), headers={}):
+    ret_val = None
+
+    try:
+        log.debug("reading simple {} ...".format(url))
+        req = Request(url, headers=headers)
+        req.get_method = lambda: 'PUT'
+        response = urlopen(req, timeout=10)
+
+        if response.code == 200:
+            ret_val = response.read()
+    except HTTPError as e:
+        log.error("The server can't fulfill the PUT {} request. [reason: {}]".format(url, e.reason))
+    except URLError as e:
+        log.error("Unable to reach the server for PUT {} request. [reason: {}]".format(url, e.reason))
+    except Exception as e:
+        log.error("There was an error of PUT url {}: {}".format(url, e.__str__()))
+    finally:
+        try:
+            response.close()
+        except NameError:
+            pass
+
+    return ret_val
+
+
 def get_gcp_metadata(url, path_params, log=logging.getLogger('spotinst-agent')):
     try:
         url = url + path_params
@@ -69,7 +86,7 @@ def get_gcp_metadata(url, path_params, log=logging.getLogger('spotinst-agent')):
         #req.add_header('Content-Type', 'application/json')
         req.add_header('Metadata-Flavor','Google')
         response = urlopen(req, timeout=10)
-        return response.read().decode('utf-8')
+        return response.read()
     except HTTPError as e:
         log.error("The server can't fulfill the GET {} request. [reason: {}]".format(url, e.reason))
     except URLError as e:
@@ -89,7 +106,7 @@ def get_azure_metadata(url, log=logging.getLogger('spotinst-agent')):
         req = Request(url)
         req.add_header('Metadata', 'true')
         response = urlopen(req, timeout=10)
-        return response.read().decode('utf-8')
+        return response.read()
     except HTTPError as e:
         log.error("The server can't fulfill the GET {} request. [reason: {}]".format(url, e.reason))
     except URLError as e:
@@ -171,7 +188,7 @@ def safe_post_url(url, message, token=None, log=logging.getLogger('spotinst-agen
         log.error("Unable to reach the server for POST request. [reason: {}]".format(e.reason))
     except SSLError as e:
         log.error("Connection to API timed out, retry in next iteration : {}".format(str(e)))
-    except http.client.BadStatusLine as e:
+    except httplib.BadStatusLine as e:
         log.error("Bad status line. The URL is most likely wrong.")
     except Exception as e:
         log.error("There was an error of POST url. Exceptions: " + str(sys.exc_info()[0]))
